
PROGRAM  WNDWPTDATA

    !!***********************************************************************
    !! Version "$Id: wndwptdata.f90 216 2023-01-06 18:39:58Z coats $"
    !! EDSS/Models-3 M3TOOLS.
    !! (C) 2023 UNC Institute for the Environment.
    !! Distributed under the GNU GENERAL PUBLIC LICENSE version 2
    !! See file "GPL.txt" for conditions of use.
    !!.........................................................................
    !!  program body starts at line  112
    !!
    !!  FUNCTION:
    !!       For a user-specified pair of GRIDDED Models-3 files and
    !!       pair of variables within them, generate an ASCII file of
    !!       matched value-pairs, one per line.
    !!
    !!  PRECONDITIONS REQUIRED:
    !!       Machine with stack-allocated AUTO local variables (e.g., CRAY)
    !!
    !!  SUBROUTINES AND FUNCTIONS CALLED:
    !!       Models-3 I/O
    !!
    !!  REVISION  HISTORY:
    !!      Prototype 1/2023 by CJC
    !!***********************************************************************

    USE M3UTILIO
    USE MODGCTP
    IMPLICIT NONE

    !!...........   EXTERNAL FUNCTIONS and their descriptions:

     INTEGER :: IARGC

    !!...........   PARAMETERS and their descriptions:

    CHARACTER*16, PARAMETER :: PNAME = 'WNDWPTDATA'
    CHARACTER*16, PARAMETER :: BLANK = ' '
    CHARACTER*72, PARAMETER :: BAR   =  &
    '-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-'

    !!...........   LOCAL VARIABLES and their descriptions:

    INTEGER         LDEV        !  log-device
    INTEGER         ISTAT       !  allocation-status
    CHARACTER*256   MESG
    LOGICAL         EFLAG

    INTEGER         NCOLS1      ! number of grid columns
    INTEGER         NROWS1      ! number of grid rows
    INTEGER         NTHIK1      ! bdy thickness
    INTEGER         GDTYP1      ! grid type:  1=LAT-LON, 2=UTM, ...
    INTEGER         FTYPE1      ! file type:  gridded, custom, boundary...
    REAL*8          P_ALP1      ! first, second, third map
    REAL*8          P_BET1      ! projection descriptive
    REAL*8          P_GAM1      ! parameters.
    REAL*8          XCENT1      ! lon for coord-system X=0
    REAL*8          YCENT1      ! lat for coord-system Y=0
    REAL*8          XORIG1      ! X-coordinate origin of grid (map units)
    REAL*8          YORIG1      ! Y-coordinate origin of grid
    REAL*8          XFINL1      ! X-coordinate UR corner of grid (map units)
    REAL*8          YFINL1      ! Y-coordinate UR corner of grid
    REAL*8          XCELL1      ! X-coordinate cell dimension
    REAL*8          YCELL1      ! Y-coordinate cell dimension

    INTEGER         FTYPEPT     ! file type
    INTEGER         NPNTSIN     ! number of input point sources
    INTEGER         SDATEPT, STIMEPT    !  starting date&time for "POINTS"
                                        !  This file *should* be time independent, damn it !!

    INTEGER         NLAYSIN     ! number of layers for input-data file
    INTEGER         NVARSIN     ! number of variables for input-data file
    INTEGER         SDATEIN     ! starting date
    INTEGER         STIMEIN     ! starting time
    INTEGER         TSTEPIN     ! time step
    INTEGER         NRECSIN     ! number of records

    INTEGER         NPNTOUT    ! number of output point sources

    INTEGER         JDATE, JTIME, TSTEP, NRECS
    INTEGER         N, T, V

    CHARACTER*16    CNAME, GNAME

    CHARACTER*16    VNAMES( MXVARS3 )
    INTEGER         VTYPES( MXVARS3 )

    INTEGER,    ALLOCATABLE :: INDX( : )      !! (NPNTSIN); effective dimension NPNTOUT
    REAL,       ALLOCATABLE :: ABUF( : )      !! (NPNTSIN)...
    REAL*8,     ALLOCATABLE :: YLAT( : )
    REAL*8,     ALLOCATABLE :: XLON( : )
    REAL*8,     ALLOCATABLE :: XVAL( : )
    REAL*8,     ALLOCATABLE :: YVAL( : )

    !!***********************************************************************
    !!   begin body of program M3COMBO

    LDEV  = INIT3()
    EFLAG = .FALSE.

    WRITE( *, '( 5X, A )' ) BLANK, BAR, BLANK,                              &
'Program WNDWPOPINTS to take a SMOKE points-data file and subset it so',    &
'that all the output sources lie in the coverage of the indicated output',  &
'grid.',                                                                    &
'',                                                                         &
'PRECONDITIONS REQUIRED:',                                                  &
'     setenv  GRIDDESC        <path-name>',                                 &
'     setenv  OUTGRID         <GRIDDESC-name for the output grid>',         &
'     setenv  POINTS          <path-name for SMOKE POINTS file>',           &
'     setenv  INFILE          <path-name for  input SMOKE data file>',      &
'     setenv  OUTFILE         <path-name for output SMOKE data file>',      &
'',                                                                         &
'See URL',                                                                  &
'',                                                                         &
'   https://www.cmascenter.org/ioapi/documentation/3.1/html/AA.html#tools', &
'',                                                                         &
'Program copyright (C) 2023 UNC Institute for the Environment.',            &
'Released under Version 2 of the GNU General Public License. See',          &
'enclosed GPL.txt, or URL',                                                 &
''  ,                                                                       &
'    https://www.gnu.org/licenses/old-licenses/gpl-2.0.html',               &
''  ,                                                                       &
'Comments and questions are welcome and can be sent to'  ,                  &
'',                                                                         &
'    Carlie J. Coats, Jr.    carlie@jyarborough.com',                       &
'or',                                                                       &
'    UNC Institute for the Environment',                                    &
'    100 Europa Dr., Suite 490 Rm 405',                                     &
'    Campus Box 1105',                                                      &
'    Chapel Hill, NC 27599-1105',                                           &
'',                                                                         &
'Program version: ',                                                        &
'$Id: wndwptdata.f90 216 2023-01-06 18:39:58Z coats $',&
' '

    IF ( .NOT.GETYN( 'Continue with program?', .TRUE. ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Program aborted', 2 )
    END IF


    !!...............  Get output grid description
    !!...............  Open and get descriptions for input "points", "infile":

    CALL ENVSTR( 'OUTGRID', 'GRIDDESC name for output grid', 'WRF_FOO_CRO', GNAME, ISTAT )
    IF ( ISTAT .GT. 0 ) THEN
        EFLAG = .TRUE.
        CALL M3MESG( 'Bad environment variable "OUTGRID"' )
    END IF

    IF ( .NOT.DSCGRID( 'OUTGRID', CNAME, GDTYP1,        &      !  "standard" grid name
                  P_ALP1, P_BET1, P_GAM1,               &
                  XCENT1, YCENT1, XORIG1, YORIG1,       &
                  XCELL1, YCELL1,                       &
                  NCOLS1, NROWS1, NTHIK1 ) ) THEN

        MESG = 'Could not get grid description for "OUTGRID" in GRIDDESC file'
        EFLAG = .TRUE.
        CALL M3MESG( MESG )

    END IF


    IF ( .NOT. OPEN3( 'POINTS', FSREAD3,PNAME ) ) THEN

        MESG = 'Could not open "POINTS"'
        EFLAG = .TRUE.
        CALL M3MESG( MESG )

    ELSE IF ( .NOT. DESC3( 'POINTS' ) ) THEN

        MESG = 'Could not get file description for "POINTS"'
        EFLAG = .TRUE.
        CALL M3MESG( MESG )

    ELSE IF ( NCOLS3D .NE. 1 .OR. NLAYS3D .NE. 1 ) THEN

        MESG = 'Invalid dimensions:  not a correct "POINTS" file'
        EFLAG = .TRUE.
        CALL M3MESG( MESG )

    ELSE

        NPNTSIN = NROWS3D
        SDATEPT = SDATE3D
        STIMEPT = STIME3D
        FTYPEPT = FTYPE3D

    END IF      !!  if open3('points'...) worked
    
    IF ( .NOT. OPEN3( 'POINTS', FSREAD3,PNAME ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Could not open "POINTS"', 2 )
    ELSE IF ( .NOT. DESC3( 'POINTS' ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Could not get description for "POINTS"', 2 )
    ELSE IF ( NCOLS3D .NE. 1 .OR. NLAYS3D .NE. 1 ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Invalid dimensions:  not a correct "POINTS" file', 2 )
    ELSE IF ( .NOT.GRDCHK3( 'POINTS', P_ALP1, P_BET1, P_GAM1, XCENT1, YCENT1,   &
                            XORIG3D, YORIG3D, XCELL3D, YCELL3D,                 &
                            NLAYS3D, VGTYP3D, VGTOP3D, VGLVS3D ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Inconsistent map-proj parameters for "POINTS"', 2 )
    ELSE

        NPNTSIN = NROWS3D
        SDATEIN = SDATE3D
        STIMEIN = STIME3D
        NVARSIN = NVARS3D
        SDATEIN = SDATE3D
        STIMEIN = STIME3D
        VNAMES( 1:NVARSIN ) = VNAME3D( 1:NVARSIN )
        VTYPES( 1:NVARSIN ) = VTYPE3D( 1:NVARSIN )

    END IF      !!  if open3('points'...) worked


    !!...............  Allocate working arrays:

    ALLOCATE( INDX( NPNTSIN ),     &
              ABUF( NPNTSIN ),     &
              YLAT( NPNTSIN ),     &
              XLON( NPNTSIN ),     &
              XVAL( NPNTSIN ),     &
              YVAL( NPNTSIN ),     STAT = ISTAT )

    IF ( ISTAT .NE. 0 ) THEN
        WRITE( MESG, '( A, I10)' ) 'Buffer allocation failed:  STAT=', ISTAT
        CALL M3EXIT( PNAME, 0, 0, MESG, 2 )
    END IF


    !!...............  Read and transform latitudes, longitudes:
    !!...............  Compute index:

    IF ( .NOT.READ3( 'POINTS', 'LATITUDE', 1, SDATEPT, STIMEPT, ABUF ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Could not read "LATITUDE" from "POINTS"', 2 )
    END IF
    YLAT = DBLE( ABUF )

    IF ( .NOT.READ3( 'POINTS', 'LONGITUDE', 1, SDATEPT, STIMEPT, ABUF ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Could not read "LONGITUDE" from "POINTS"', 2 )
    END IF
    XLON = DBLE( ABUF )

    CALL XY2XY( GDTYP1,  P_ALP1, P_BET1, P_GAM1, XCENT1, YCENT1,          &
                LATGRD3, 0.0D0,   0.0D0,  0.0D0,  0.0D0,  0.0D0,          &
                NPNTSIN, XLON, YLAT, XVAL, YVAL )

    XFINL1  = XORIG1 + DBLE( NCOLS1 )*XCELL1
    YFINL1  = YORIG1 + DBLE( NROWS1 )*YCELL1
    NPNTOUT = 0
    DO N = 1, NPNTSIN

        IF ( XVAL( N ) .LT. XORIG1 ) CYCLE      !!  skip:  outside output-grid
        IF ( XVAL( N ) .GT. XFINL1 ) CYCLE
        IF ( YVAL( N ) .LT. YORIG1 ) CYCLE
        IF ( YVAL( N ) .GT. YFINL1 ) CYCLE

        NPNTOUT = NPNTOUT + 1
        INDX( NPNTOUT ) = N

    END DO

    WRITE( MESG, '( A, I10 )' ) 'Outout window NPNTS=', NPNTOUT
    CALL M3MESG( MESG )


    !!...............  Create "OUTFILE" using description borrowed from "INFILE" and edited:

    NROWS3D = NPNTOUT
    IF ( .NOT.OPEN3( 'OUTFILE', FSUNKN3, PNAME ) ) THEN
        CALL M3EXIT( PNAME, 0, 0, 'Could not open/create "OUTFILE"', 2 )
    END IF


    !!...............  Process time step sequence

    EFLAG = .FALSE.
    JDATE = SDATEIN
    JTIME = STIMEIN
    TSTEP = TSTEPIN

    DO T = 1, NRECSIN

        DO V = 1, NVARSIN

            IF (      VTYPES( V ) .EQ. M3REAL ) THEN

                CALL WNDWREAL( VNAMES(V), EFLAG )

            ELSE IF ( VTYPES( V ) .EQ. M3INT  ) THEN

                CALL WNDWINT( VNAMES(V), EFLAG )

            ELSE IF ( VTYPES( V ) .EQ. M3DBLE ) THEN

                CALL WNDWDBLE( VNAMES(V), EFLAG )

            ELSE

                WRITE( MESG, '( A,I10 )' ) 'Unsupported variable type', VTYPES( V )
                CALL M3MESG( MESG )
                EFLAG = .TRUE.

            END IF

        END DO

        CALL NEXTIME( JDATE, JTIME, TSTEP )

    END DO


    !!...............  Completion


    IF ( EFLAG ) THEN
        MESG  = 'Failure in program'
        ISTAT = 2
    ELSE
        MESG  = 'Success in program'
        ISTAT = 0
    END IF

    CALL M3EXIT( PNAME, 0, 0, MESG, ISTAT )


CONTAINS    !!-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


    SUBROUTINE WNDWREAL( VNAME, EFLAG )

        CHARACTER(16), INTENT(IN   ) :: VNAME
        LOGICAL,       INTENT(INOUT) :: EFLAG

        INTEGER     L, N, P

        REAL        IBUF( NPNTSIN,  NLAYSIN )
        REAL        OBUF( NPNTOUT, NLAYSIN )

        IF ( .NOT.READ3( 'INFILE', ALLAYS3, JDATE, JTIME, IBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        DO L = 1, NLAYSIN
        DO N = 1, NPNTOUT
            P = INDX( N )
            OBUF( N,L ) = IBUF( P,L )
        END DO
        END DO

        IF ( .NOT.WRITE3( 'OUTFILE', JDATE, JTIME, OBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        RETURN

    END SUBROUTINE WNDWREAL


    !!-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


    SUBROUTINE WNDWINT( VNAME, EFLAG )

        CHARACTER(16), INTENT(IN   ) :: VNAME
        LOGICAL,       INTENT(INOUT) :: EFLAG

        INTEGER     L, N, P

        INTEGER     IBUF( NPNTSIN,  NLAYSIN )
        INTEGER     OBUF( NPNTOUT, NLAYSIN )

        IF ( .NOT.READ3( 'INFILE', ALLAYS3, JDATE, JTIME, IBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        DO L = 1, NLAYSIN
        DO N = 1, NPNTOUT
            P = INDX( N )
            OBUF( N,L ) = IBUF( P,L )
        END DO
        END DO

        IF ( .NOT.WRITE3( 'OUTFILE', JDATE, JTIME, OBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        RETURN

    END SUBROUTINE WNDWINT


    !!-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


    SUBROUTINE WNDWDBLE( VNAME, EFLAG )

        CHARACTER(16), INTENT(IN   ) :: VNAME
        LOGICAL,       INTENT(INOUT) :: EFLAG

        INTEGER     L, N, P

        REAL*8      IBUF( NPNTSIN,  NLAYSIN )
        REAL*8      OBUF( NPNTOUT, NLAYSIN )

        IF ( .NOT.READ3( 'INFILE', ALLAYS3, JDATE, JTIME, IBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        DO L = 1, NLAYSIN
        DO N = 1, NPNTOUT
            P = INDX( N )
            OBUF( N,L ) = IBUF( P,L )
        END DO
        END DO

        IF ( .NOT.WRITE3( 'OUTFILE', JDATE, JTIME, OBUF ) ) THEN
            EFLAG = .FALSE.
            RETURN
        END IF

        RETURN

    END SUBROUTINE WNDWDBLE


END PROGRAM WNDWPTDATA

