
        SUBROUTINE CLDFRAC2 ( IM, JM, KM, DX, PSTAR, PBL,
     &                    SIGH, SIGF, PTOP, P00, TLP, TS0, ROVG, RDRY,
     &                       NCOLS, NROWS, NLAYS,
     &                       PRES, RH, CFRACL, CFRACM, CFRACH )

C***********************************************************************
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999-202 MCNC
C and (C) 2004-2009 Baron Advanced Metorological Systems, LLC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................
C  subroutine body starts at line  77
C
C  FUNCTION:
C       Compute the cross point cloud fractions for the following
C       pressure groupings:
C               low (970-800 mb), 
C               middle (800-450 mb),
C               high level (above 450 mb) 
C       using the MM5 sfcrad.F relative humidity based algorithm,
C       from 3D pressure and relative humidity, 
C
C  PRECONDITIONS REQUIRED:
C       PRES and RH already computed in I/O API order by MM2THERM3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype 12/2001 by Carlie J. Coats, Jr., MCNC EMC.
C       Version   08/2009 by CJC:  new cloud algorithm
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            DX      !  MM5 cell-size
        REAL            PSTAR( IM, JM ) !  MM5 pstar array
        REAL            PBL  ( IM, JM ) !  MM5 PBL-height array
        REAL            SIGH( KM )      !  MM5 SIGMA-H
        REAL            SIGF( KM+1 )    !  MM5 SIGMA-F
        REAL            PTOP		!  MM5 ptop (cb)
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            TLP		!  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		!  MM5 ref SL temp (K)
        REAL            ROVG		!  MM5 R/G
        REAL            RDRY		!  MM5 dry gas constant
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        REAL            PRES( NCOLS, NROWS, NLAYS ) !  pressure (cb)
        REAL            RH( NCOLS, NROWS, NLAYS ) !  rh, 2-phase version
        REAL            CFRACL( NCOLS, NROWS ) !  cloud fraction
        REAL            CFRACM( NCOLS, NROWS ) !  rh, liq. water version
        REAL            CFRACH( NCOLS, NROWS ) !  rh, liq. water version

C...........   PARAMETERs

        REAL, PARAMETER :: MBTOPA  =    1.0E2
        REAL, PARAMETER :: PRESBOT =  970.0 * MBTOPA
        REAL, PARAMETER :: PRESMID =  800.0 * MBTOPA
        REAL, PARAMETER :: PRESTOP =  450.0 * MBTOPA

        REAL, PARAMETER :: CLDRAD   = 1000.0    !  size scale for cloud
        REAL, PARAMETER :: CPA  = 1.0E3          !  conversion factor, Pa to cb
        REAL, PARAMETER :: RHC1 = 0.98


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K      !  MM5 subscripts
        INTEGER         I0, J0, K0   !  MM5 offsets
        INTEGER         KPBL, ROW0, COL0, LAY0
        REAL            RFAC1, RFAC2, FRAC, DPRES, RHC
        REAL            CCOV
        REAL            SG1, PSFC, PRF0, ZT, ZF
        REAL            DP0          !  1/p0
        REAL            DR           !  1/rdry
        REAL            LPS, LPF, RHL


C***********************************************************************
C   begin body of subroutine  MM2DPT

        !!  scale-dependent modification of relative humidity
        !!  threshold for cloud formation:
        !!  RHC_new = RHC_mm5 + RFAC1 * ( 1.0 - RHC_mm5 )
        !!  then simplify the algebra, where:
        !!      RFAC1 --> 0 as DX --> infinity
        !!      RFAC1 --> 1 as DX --> 0
        !!      RFAC1 = 0.5 at DX = CLDRAD
	
        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DP0 = 1.0 / P00
        DR  = 1.0 / RDRY
        K   = K0 - 1

        RFAC1 = CLDRAD**2 / ( CLDRAD**2 + DX**2 )
        RFAC2 = 1.0 - RFAC1

        DO  R = 1, NROWS
        DO  C = 1, NCOLS

            I = I0 + R
            J = J0 + C

            PSFC = CPA*( PSTAR( I,J ) + PTOP )
            LPS  = ALOG( PSFC * DP0 )
            ZT   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )

            !!  Find PBL-layer

            DO  L = 2, NLAYS
                PRF0 = CPA*( PSTAR( I,J )*SIGF( K ) + PTOP )
                LPF  = ALOG( PRF0 * DP0 )
                ZF   = -ROVG*LPF*( 0.5*TLP*LPF + TS0 ) - ZT
                IF ( PBL( I,J )  .LE. ZF ) THEN
                    KPBL = L
                    EXIT
                END IF
            END DO
            
            CFRACL( C,R ) = 0.0
            CFRACM( C,R ) = 0.0
            CFRACH( C,R ) = 0.0

            DO L = 1, KPBL

                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_

                IF ( RHL .GT. RHC1 ) THEN

                    CCOV = 0.34*( RH( C,R,L ) - RHC1 )/( 1.0- RHC1 )
                    CCOV = MIN( CCOV, 1.0 )

                    IF      (  PRES( C,R,L ) .LE. PRESTOP ) THEN
                        CFRACH(C,R) = MAX( CFRACH(C,R), CCOV )
                    ELSE IF (  PRES( C,R,L ) .LE. PRESMID ) THEN
                        CFRACM(C,R) = MAX( CFRACM(C,R), CCOV )
                    ELSE IF (  PRES( C,R,L ) .LE. PRESBOT ) THEN
                        CFRACL(C,R) = MAX( CFRACL(C,R), CCOV )
                    END IF

                END IF

            END DO

            DPRES = 1.0 / PRES( C,R,KPBL )
            DO L = KPBL+1, NLAYS

                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_

                SG1 = DPRES * PRES( C,R,L )
                RHC = 1.0 - ( 2.0 * SG1 * (1.0-SG1) *
     &                            (1.0 + 1.732*(SG1-0.5)) )
                RHC = RFAC2 * RHC + RFAC1

                IF ( RHL .GT. RHC ) THEN

                    CCOV = ( ( RHL - RHC ) / ( 1.0 - RHC ) )**2
                    CCOV = MIN( CCOV, 1.0 )

                    IF      (  PRES( C,R,L ) .LE. PRESTOP ) THEN
                        CFRACH(C,R) = MAX( CFRACH(C,R), CCOV )
                    ELSE IF (  PRES( C,R,L ) .LE. PRESMID ) THEN
                        CFRACM(C,R) = MAX( CFRACM(C,R), CCOV )
                    ELSE IF (  PRES( C,R,L ) .LE. PRESBOT ) THEN
                        CFRACL(C,R) = MAX( CFRACL(C,R), CCOV )
                    END IF

                END IF

            END DO
            
        END DO
        END DO

        RETURN
        END

