
        SUBROUTINE CLDFRAC3( IM, JM, KM, DX, PSTAR, PBL, XLAND, TER,
     &                       SIGH, PTOP, P00, TLP, TS0, ROVG, RDRY,
     &                       NCOLS, NROWS, NLAYS, COL0, ROW0,
     &                       PRES, RH, CFRACL, CFRACM, CFRACH )

C***********************************************************************
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999-202 MCNC
C and (C) 2004-2010 Baron Advanced Metorological Systems, LLC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................
C  subroutine body starts at line  97
C
C  FUNCTION:
C       Compute the cross point cloud fractions for the following
C       pressure groupings:
C               low (970-800 mb),
C               middle (800-450 mb),
C               high level (above 450 mb)
C       using a variant on the MM5 sfcrad.F relative humidity based
C       algorithm from 3D pressure and relative humidity,
C
C  PRECONDITIONS REQUIRED:
C       PRES and RH already computed in I/O API order by MM2THERM3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype 12/2001 by Carlie J. Coats, Jr., MCNC EMC.
C       Version   08/2009 by CJC:  new cloud algorithm
C       Version   08/2010 by CJC:  newer cloud algorithm with ocean
C       adjustment, consistent with MM2HTR9()
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            DX      !  MM5 cell-size
        REAL            PSTAR( IM, JM ) !  MM5 pstar array
        REAL            PBL  ( IM, JM ) !  MM5 PBL-height array
        REAL            XLAND( IM, JM ) !  MM5 xland array
        REAL              TER( IM, JM ) !  MM5 terrain array == geopotential ht
        REAL            SIGH( KM )      !  MM5 SIGMA-H
        REAL            PTOP            !  MM5 ptop (cb)
        REAL            P00             !  MM5 reference SL-pres (Pa)
        REAL            TLP             !  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0             !  MM5 ref SL temp (K)
        REAL            ROVG            !  MM5 R/G
        REAL            RDRY            !  MM5 dry gas constant
        INTEGER         NCOLS           !  output dimension
        INTEGER         NROWS           !  "
        INTEGER         NLAYS           !  "
        INTEGER         COL0            !  starting subscript for output window
        INTEGER         ROW0            !  "
        REAL              PRES( NCOLS, NROWS, NLAYS ) !  pressure (cb)
        REAL                RH( NCOLS, NROWS, NLAYS ) !  rh, 2-phase version
        REAL            CFRACL( NCOLS, NROWS ) !  cloud fraction
        REAL            CFRACM( NCOLS, NROWS ) !  "
        REAL            CFRACH( NCOLS, NROWS ) !  "

C...........   PARAMETERs

        REAL, PARAMETER :: MBTOPA  =    1.0E2
        REAL, PARAMETER :: PRESBOT =  970.0 * MBTOPA
        REAL, PARAMETER :: PRESMID =  800.0 * MBTOPA
        REAL, PARAMETER :: PRESTOP =  450.0 * MBTOPA
        REAL, PARAMETER :: CPA     =    1.0E3          !  conversion factor, Pa to cb
        REAL, PARAMETER :: G       =    9.8
        REAL, PARAMETER :: RHC1    =    0.98


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         C, R, L         !  IOAPI col subscript
        INTEGER         LBOT, LMID, LTOP, LPBL
        INTEGER         I, J, K      !  MM5 subscripts
        INTEGER         I0, J0, K0   !  MM5 offsets
        REAL            RFAC1, RFAC2, FRAC, DPRES
        REAL            SG1, PRF0, ZT, ZH
        REAL            DP0          !  1/p0
        REAL            DR           !  1/rdry
        REAL            PL, LPF
        REAL            RAT, RHCP, RHC, RHCH, RHCW, RHL
        REAL            CCOV, CTOP, CMID, CBOT


C***********************************************************************
C   begin body of subroutine  MM2DPT

        I0 = ROW0 - 1           !  offsets for I/O API grid within MM5
        J0 = COL0 - 1
        K0 = KM + 1

        DP0 = 1.0 / P00
        DR  = 1.0 / RDRY
        K   = K0 - 1

        RAT  = MIN( MAX( ( 45000.0 - DX ) / 42000.0, 0.0 ), 1.0 )
        RHC  = 0.75 + 0.17 * RAT**2     !  =0.75 if dx>45km, =0.92 if dx< 3km, quadratic between
        RHCH = 0.8 * RHC + 0.2          !  for high clouds
        RHCW = 0.5 * RHC + 0.5          !  for over-water low clouds

        DO  R = 1, NROWS
        DO  C = 1, NCOLS

            I = I0 + R
            J = J0 + C

            ZT   = TER( I,J ) / G     !!  terrain height:: geopotential / G

            !!  Find PBL-layer, cloud-layer levels

            LPBL = 1
            LBOT = 1
            LMID = 2
            LTOP = 3

            DO  L = 1, NLAYS

                K    = K0 - L
                PRF0 = CPA*( PSTAR( I,J )*SIGH( K ) + PTOP )
                LPF  = ALOG( PRF0 * DP0 )
                ZH   = -ROVG*LPF*( 0.5*TLP*LPF + TS0 ) - ZT

                IF ( ZH .LE. PBL( I,J ) )  LPBL = L

                PL = PRES( C,R,L )

                IF ( PL .GT. PRESTOP ) LTOP = L+1
                IF ( PL .GT. PRESMID ) LMID = L+1
                IF ( PL .GT. PRESBOT ) LBOT = L+1

            END DO

            LTOP = MIN( LTOP, NLAYS )
            LMID = MIN( LMID, LTOP-1 )
            LBOT = MIN( LBOT, LMID-1 )
            LPBL = MIN( LPBL, NLAYS-1 )

            CTOP = 0.0
            CMID = 0.0
            CBOT = 0.0

            DO   L = LBOT, LPBL            !! L-loop for cloud frac below PBL

                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_

                IF ( RHL .LE. RHC1 ) CYCLE

                CCOV = ( RHL - RHC1 ) / ( 1.0 - RHC1 )
                IF      (  L .GE. LTOP ) THEN
                    CTOP = MAX( CCOV, CTOP )
                ELSE IF (  L .GE. LMID ) THEN
                    CMID = MAX( CCOV, CMID )
                ELSE
                    CBOT = MAX( CCOV, CBOT )
                END IF      !  if OK pressure for cloud calc.

            END DO                !! end of L-loop for cloud frac

            IF ( LPBL .GE. LBOT-1 ) THEN
                L    = LPBL+1
                RHL  = 0.01 * RH( C,R,L )                !  humidity _fraction_
                RHCP = 0.5 * ( RHC1 + RHC )
                IF ( RHL .GT. RHCP ) THEN
                    CCOV = ( RHL - RHCP ) / ( 1.0 - RHCP )
                    IF      (  L .GE. LTOP ) THEN
                        CTOP = MAX( CCOV, CTOP )
                    ELSE IF (  L .GE. LMID ) THEN
                        CMID = MAX( CCOV, CMID )
                    ELSE
                        CBOT = MAX( CCOV, CBOT )
                    END IF      !  if OK pressure for cloud calc.
                END IF
            END IF              !  if lpbl > lbot

            LTOP = MAX( LTOP, LPBL+4 )
            LMID = MAX( LMID, LPBL+3 )
            LBOT = MAX( LBOT, LPBL+2 )

            DO L = LTOP, NLAYS            !! L-loop for cloud frac above PBL
                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_
                IF ( RHL .LT. RHCH ) CYCLE
                CCOV = ( RHL - RHCH ) / ( 1.0 - RHCH )
                CTOP = MAX( CCOV, CTOP )
            END DO                !! end of L-loop for cloud frac

            DO L = LMID, LTOP-1           !! L-loop for cloud frac above PBL
                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_
                IF ( RHL .LT. RHC ) CYCLE
                CCOV = ( RHL - RHC ) / ( 1.0 - RHC )
                CMID = MAX( CCOV, CMID )
            END DO                !! end of L-loop for cloud frac

            IF ( XLAND( I,J ) .GT. 1.5 ) THEN
                RHCP = RHCW
            ELSE
                RHCP = RHC
            END IF

            DO L = LBOT, LMID-1           !! L-loop for cloud frac above PBL
                RHL = 0.01 * RH( C,R,L )                !  humidity _fraction_
                IF ( RHL .LT. RHCP ) CYCLE
                CCOV = ( RHL - RHCP) / ( 1.0 - RHCP )
                CBOT = MAX( CCOV, CBOT )
            END DO                !! end of L-loop for cloud frac

            CFRACL( C,R ) = CBOT
            CFRACM( C,R ) = CMID
            CFRACH( C,R ) = CTOP

        END DO
        END DO

        RETURN
        END

