
        SUBROUTINE  MM2SLP( IM, JM, KM, 
     &                      TA, PP, PST, SIGH, PTOP, TER,
     &                      NCOLS, NROWS, NLAYS,
     &                      COL0, ROW0, LAY0, 
     &                      SLP)

C***********************************************************************
C Version "%W% %P% %G% %U%
C-----------------------------------------------------------------------
C  subroutine body starts at line  75
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	sea level pressure for the window specified in terms of grid-window 
C       bounds to I/O API order, using
C       TA, Pprime, Pstar, SIGH, Ptop, TER from MM5. Returns Sea Level Pressure
C       in mb....
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  Jan. 25, 1999 by Carlie J. Coats, Jr., MCNC
C       Revised/Validated 1/25/99   John McHenry, MCNC
C       Version 7/2/2010 CJC: format-bug fix
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            PP( IM, JM, KM )        !  MM5 P-prime (CB*PA) 
        REAL            PST( IM, JM )           !  MM5 pstar array (CB)
        REAL            TER( IM, JM )           !  MM5 terrain array == geopotential height 
                                                !  (i.e. actual height times g)
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop (input as CB)
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            SLP( NCOLS, NROWS ) ! sea-level pressure.... 

C...........   PARAMETERs

        REAL            GAMMA                   !  MM5 gamma
        REAL            RDGAS                   !  MM5 r
        REAL            G                       !  MM5 g
        REAL            TC      ! critical temp in Psfc/Pslv
        REAL            PCONST  ! pressure above sfc defining
                                ! used for estimating Tsfc.
        REAL            XTERM   ! GAMMA*R/G
        REAL            RDG     ! R/G
        REAL            TWODR   ! 2.0/R
        REAL            PATOMB
        REAL            CBTOPA
        REAL            MBTOPA

        PARAMETER( GAMMA = 6.5E-3,
     &             RDGAS = 287.04,
     &             G     = 9.8,
     &             TC    = 273.16 + 17.5,
     &             PCONST= 100.,            !! mb = 1hPA
     &             XTERM = GAMMA*RDGAS/G,
     &             RDG   = RDGAS/G,
     &             TWODR = 2.0/RDGAS,
     &             PATOMB = 1.E-2,
     &             CBTOPA = 1.E3,
     &             MBTOPA = 1.E2)

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        
        REAL            DFAC

        INTEGER   KLO, KHI
        REAL      T0, PK, PL, TS, DPHI, PRAT
        REAL      PHI, PLO, THI, TLO, TL, SFP
        REAL      TBAR, HL

        CHARACTER*256  MESG

C***********************************************************************
C   begin body of subroutine  MM2SLP

        I0 = ROW0 - 1                   !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C

                DFAC = 1.0 / PST( I,J )         ! factor for decoupling

C     ... Compute pressure at the surface -- (following INTERP,
C         this is an estimate with the assumption that PP at
C         the lowest half sigma level is equivalent to PP at 
C         the model surface where full sigma = 1. Note: KF has
C         an extrapolation approach to estimate PP at SIGF=1.
C         WORK in Pa and at the end convert to mb.
C     
                SFP = CBTOPA*(PST(I,J) + PTOP) + DFAC*PP(I,J,KM)  !! all in Pa

C     ... Compute pressure at PCONST MB above surface (PL)
C         Note:  must decouple PP...watch units, work in mb...

                PL  = SFP - PCONST*MBTOPA       !! in Pa
                KHI = 1

                PHI = CBTOPA*(SIGH(KHI)*PST(I,J) + PTOP) + 
     &               DFAC*PP(I,J,KHI)

                KLO = KM

                PLO = CBTOPA*(SIGH(KLO)*PST(I,J) + PTOP) +
     &                DFAC*PP(I,J,KLO)

                IF( ( PLO .GE. PL ) .AND. ( PHI .LT. PL ) ) THEN
         
31                  CONTINUE    !  binary search for PL
                        K  = ( KLO + KHI ) / 2
                        PK = CBTOPA*(SIGH(K)*PST(I,J) + PTOP) +
     &                       DFAC*PP(I,J,K)
                        IF ( PK .LT. PL ) THEN
                            KHI = K
                            PHI = PK
                        ELSE
                            KLO = K
                            PLO = PK
                        END IF
                        IF ( KHI .LT. KLO-1 ) THEN
                            GO TO  31
                        END IF

                ELSE

                    WRITE( MESG ,'( A, 2I5 )' ) 'I,J  = ',I,J
                    CALL M3MSG2( MESG )
                    WRITE( MESG ,'( 2( A, 2( 1PE15.6, 2X ) ) )' )
     &                    'PLO,PHI= ',PLO, PHI,
     &                    'PSFC,PL= ', SFP, PL
                    MESG = 'PL not trapped between 1 and KM'
                    CALL M3EXIT( 'MCPL:MM2SLP', 0,0, MESG, 2 )

                END IF

C     ... get TL temperature at pl, extrapolate T at surface (TS)
C         and T at sea level (T0) with 6.5 K/km lapse rate

                DPHI = 1.0 / PHI
                PRAT = SFP / PL
                TLO  = DFAC*TA(I,J,KLO)  !  decoupled LO temperature
                THI  = DFAC*TA(I,J,KHI)  !  decoupled HI temperature
                TL   = THI - (THI - TLO)*ALOG(PL*DPHI)/ALOG(PLO*DPHI)
                TS   = TL * PRAT**XTERM
                TBAR = (TS + TL)*0.5
                HL   = TER(I,J)/G + RDG*TBAR*ALOG( PRAT )  !! divide out G from TER
                T0   = TL + GAMMA*HL                       !! Sea-level estimated Temp.

C     ... Correct sea level temperature if too hot

                IF ( T0 .GE. TC ) THEN
                    IF( TS .LE. TC ) THEN
                        T0 = TC
                    ELSE
                        T0 = TC - 0.005*( TS - TC )**2
                    END IF 
                END IF

C     ... Compute sea level pressure

                SLP(C,R) = PATOMB*SFP*EXP( TWODR*TER(I,J)/( TS + T0 ) )

11          CONTINUE
22      CONTINUE

        RETURN
        END
