
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.
C Copyright (C) 1998-2002 MCNC and Carlie J. Coats, Jr., and
C (C) 2003 Baron Advanced Meteorological Systems
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMXUSDVM( DX, IM, JM, KM, UM, VM, PM, FM,
     &                        MSFX, MSFD,
     &                        NCOLS, NROWS, NLAYS,
     &                        COL0, ROW0, LAY0, 
     &                        UX, VX, SPD, HDIV, AVOR, RVOR )

C***********************************************************************
C  subroutine body starts at line  117
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-point wind
C       components, wind-speed, divergence and vorticity sub-arrays
C       specified in terms of grid-window bounds to I/O API order
C       from U and V wind component arrays UM and VM and 
C       PSTAR-array PM found in standard MM5 subscript order.
C       Computes both relative vorticity RVOR (with respect to
C       Cartesian wind components only) and also absolute vorticity AVOR
C       which also has a coriolis term.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       starting and ending <column,row,level> ranges <CA,RA,LA> and
C       <CZ,RZ,LZ> are in-range for input array AM.
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C       Adapted 4/2001 by CJC from MMXUSDV() to output both 
C       relative and absolute vorticity
C
C       Version  9/2002 by CJC:  eliminate MALLOC()
C
C       Version  5/2003 by CJC:  speed, derivatives use map-factors
C***********************************************************************

      IMPLICIT NONE


C...........   INCLUDES:

#include "parame.incl"

C...........   ARGUMENTS and their descriptions:

        REAL            FACTOR  !  conversion factor
        REAL            DX      !  cell size
        INTEGER         IM      !  I (north-south) dimension for UM,VM
        INTEGER         JM      !  J (east-west)   dimension for UM,VM
        INTEGER         KM      !  K (top-bottom)  dimension for UM,VM
        REAL            UM( IM, JM, KM )        !  MM5-order input array
        REAL            VM( IM, JM, KM )        !  MM5-order input array
        REAL            QDOT( IM, JM, KM )      !  MM5-order input array
        REAL            PM( IM, JM )            !  MM5-order PSTAR array
        REAL            FM( IM, JM )            !  MM5-order F array
        REAL            MSFX( IM, JM )          !  X-point inverse-MSF array
        REAL            MSFD( IM, JM )          !  D-point inverse-MSF array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL             UX( NCOLS, NROWS, NLAYS )
        REAL             VX( NCOLS, NROWS, NLAYS )
        REAL            SPD( NCOLS, NROWS, NLAYS )
        REAL           HDIV( NCOLS, NROWS, NLAYS )	!  horiz divergence
        REAL           TDIV( NCOLS, NROWS, NLAYS )	!  3-D divergence
        REAL           AVOR( NCOLS, NROWS, NLAYS )	!  abs vorticity
        REAL           RVOR( NCOLS, NROWS, NLAYS )	!  rel vorticity

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		!  IOAPI col subscript
        INTEGER         I, J, K         !  MM5 subscripts
        INTEGER         I0, J0, K0      !  MM5 offsets

#ifdef LANGUAGE_FORTRAN90
        REAL            D( NCOLS+1,NROWS+1 )    !  decouple factor
        REAL            M( NCOLS+1,NROWS+1 )    !  derivative factor
        REAL            S( NCOLS+1,NROWS+1 )    !  coupled dot-pt spd**2
        REAL            U( NCOLS+1,NROWS+1 )    !  decoupled UM
        REAL            V( NCOLS+1,NROWS+1 )    !  decoupled VM
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL            D( MJX,MIX )    !  decouple factor
        REAL            M( MJX,MIX )    !  derivative factor
        REAL            S( MJX,MIX )    !  coupled dot-pt spd**2
        REAL            U( MJX,MIX )    !  decoupled UM
        REAL            V( MJX,MIX )    !  decoupled VM
#endif    /* ifndef LANGUAGE_FORTRAN90  */

        REAL            DUDX
        REAL            DUDY
        REAL            DVDX
        REAL            DVDY
        REAL            FAC

C***********************************************************************
C   begin body of subroutine  MMXUSDVM
	
        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        FAC  = 0.5 / DX

        DO  R = 1, NROWS + 1    !  decoupling-factor, map factor
        DO  C = 1, NCOLS + 1
            I = I0 + R
            J = J0 + C
            D( C,R ) = 4.0 / ( PM( I-1,J-1 )
     &                       + PM( I  ,J-1 )
     &                       + PM( I-1,J   )
     &                       + PM( I  ,J   ) )
            M( C,R ) = FAC / MSFX(I,J)**2
        END DO
        END DO

        K0 = KM - LAY0 + 2

        DO  L = 1, NLAYS

            K     = K0 - L

            DO  R = 1, NROWS + 1    !  compute dot-point decoupled winds
            DO  C = 1, NCOLS + 1
                I = I0 + R
                J = J0 + C
                U( C,R ) =  D( C,R ) * UM( I,J,K )
                V( C,R ) =  D( C,R ) * VM( I,J,K )
                S( C,R ) =  U( C,R )**2 + V( C,R )**2
            END DO
            END DO

            DO  R = 1, NROWS    !  compute cross point winds, wind-speed
            DO  C = 1, NCOLS
                UX( C,R,L ) = 0.25 * ( U( C+1,R   ) + U( C,R   ) 
     &                               + U( C+1,R+1 ) + U( C,R+1 ) )
                VX( C,R,L ) = 0.25 * ( V( C+1,R   ) + V( C,R   ) 
     &                               + V( C+1,R+1 ) + V( C,R+1 ) )
                SPD( C,R,L ) = 0.5 * SQRT( S( C  , R   ) +
     &                                     S( C+1, R   ) +
     &                                     S( C,   R+1 ) +
     &                                     S( C+1, R+1 ) )
            END DO
            END DO

            DO  R = 1, NROWS + 1    !  re-scale by map-factors
            DO  C = 1, NCOLS + 1
                I = I0 + R
                J = J0 + C
                U( C,R ) =  MSFD( I,J ) * U( C,R )
                V( C,R ) =  MSFD( I,J ) * V( C,R )
            END DO
            END DO

            DO  R = 1, NROWS        !  re-scaled decoupled cross-point speed
            DO  C = 1, NCOLS
                I = I0 + R
                J = J0 + C
                DUDX = M( C,R ) * ( U( C+1,R   ) - U( C,R   ) 
     &                            + U( C+1,R+1 ) - U( C,R+1 ) )
                DVDX = M( C,R ) * ( V( C+1,R   ) - V( C,R   ) 
     &                            + V( C+1,R+1 ) - V( C,R+1 ) ) 
                DUDY = M( C,R ) * ( U( C+1,R+1 ) - U( C+1,R ) 
     &                            + U( C  ,R+1 ) - U( C,R   ) ) 
                DVDY = M( C,R ) * ( V( C+1,R+1 ) - V( C+1,R ) 
     &                            + V( C  ,R+1 ) - V( C,R   ) ) 
                HDIV( C,R,L ) = DUDX + DVDY
                RVOR( C,R,L ) = DVDX - DUDY
                AVOR( C,R,L ) = RVOR( C,R,L ) + FM( I,J )
            END DO
            END DO

        END DO        !!  end loop on levels L

        RETURN
        END

